#include "../Framework/Framework.h"
#include "../Framework/Util/ProceduralTerrain.h"
#include "FileMapper.h"

//#include <iostream>

//using namespace std;

class MainApp : public OpenGLApp {
protected:
	TextureID ground;
	TextureID skyPosX, skyNegX, skyPosY, skyPosZ, skyNegZ;

	ProceduralTerrain *terrain;

public:
	bool init();
	bool load();
	bool unload();
	bool drawFrame();

	void setKey( UINT key, bool value);
	void resetCamera();
	void selectPixelFormat(int *&iAtt, float *&fAtt);
	void processMovement( const Vertex& newPosition);

private :
    FileMapper m_mapper;
	char m_position_string[256];
};

void MainApp::resetCamera(){
	position = Vertex(0,100,0);
	Wx = 0;
	Wy = 1.0f;
	Wz = 0;
}

void MainApp::selectPixelFormat(int *&iAtt, float *&fAtt){
	*iAtt++ = RED_BITS;
	*iAtt++ = 8;
	*iAtt++ = GREEN_BITS;
	*iAtt++ = 8;
	*iAtt++ = BLUE_BITS;
	*iAtt++ = 8;
	*iAtt++ = ALPHA_BITS;
	*iAtt++ = 0;

	*iAtt++ = DEPTH_BITS;
	*iAtt++ = 24;
	*iAtt++ = STENCIL_BITS;
	*iAtt++ = 0;
}

const char* map_addr;

float terrainFunc(const float x, const float z){
    int ix=(int)(x/8.0f)+45*128;
    int iz=-(int)(z/8.0f)+22*128;
    if ( ix<0 || iz<0 || ix>=90*128 || iz>=44*128)
    {
        //cerr<<"Out of bounds "<<x<<" "<<z<<endl;
        return 0.0f;
    }
    int offset=(ix+iz*90*128)*2;
    return (float)( ( map_addr[offset]<<8)|(map_addr[offset+1]&0xff))/30.0f;
}

void MainApp::setKey( UINT key, bool value)
{
	if ( value && key==demoModeKey)
	{
		setShowDebugInfo( ! showDebugInfo);
	}
	else
#if defined(WIN32)
		RenderApp::setKey( key, value);
#else
		OpenGLApp::setKey( key, value);
#endif
}

void MainApp::processMovement( const Vertex& newPosition)
{
	float elev=terrainFunc( newPosition.x, newPosition.z);
	if ( elev>newPosition.y-8.0f)
	{
		Vertex new_vertex( newPosition);
		new_vertex.y=elev+8.0f;
#if defined(WIN32)
		RenderApp::processMovement( new_vertex);
	}
	else
		RenderApp::processMovement( newPosition);
#else
		OpenGLApp::processMovement( new_vertex);
	}
	else
		OpenGLApp::processMovement( newPosition);
#endif

	sprintf( m_position_string, "%.1f E, %.1f N %d m", newPosition.x/1024.0f+45.0f, -22.0f+newPosition.z/1024.0f, (int)(elev*30.0f));
}

bool MainApp::init(){
	m_position_string[0]=0;
	m_mapper.open( "topo.img", 90*44*128*128*2);
	map_addr=(const char*)m_mapper.getMapped();
    speed=256;
	return true;
}

bool MainApp::load(){
	ground = renderer->addTexture("mars_text.jpg");

	skyNegX = renderer->addTexture("../Textures/CubeMaps/FogSky/negx.jpg", TEX_CLAMP);
	skyPosX = renderer->addTexture("../Textures/CubeMaps/FogSky/posx.jpg", TEX_CLAMP);
	skyPosY = renderer->addTexture("../Textures/CubeMaps/FogSky/posy.jpg", TEX_CLAMP);
	skyNegZ = renderer->addTexture("../Textures/CubeMaps/FogSky/negz.jpg", TEX_CLAMP);
	skyPosZ = renderer->addTexture("../Textures/CubeMaps/FogSky/posz.jpg", TEX_CLAMP);

	textTexture  = renderer->addTexture("../Textures/font2.dds");

	projection.loadProjectionMatrix(90 * PI / 180, 0.75f, 1, 4000);

	terrain = new ProceduralTerrain(terrainFunc, 1024.0f, 128, 4, 140);

	return true;
}

bool MainApp::unload(){
	delete terrain;

	return true;
}

bool MainApp::drawFrame(){

	glMatrixMode(GL_PROJECTION);
	glLoadMatrixf(projection);

	modelView.loadRotateZXY(-Wx, -Wy, -Wz);

    glMatrixMode(GL_MODELVIEW);
	glLoadMatrixf(modelView);

	glClearColor(58.0f / 255, 68.0f / 255, 184.0f / 255, 1.0f);
	glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
	glDepthFunc(GL_LEQUAL);

	glDisable(GL_DEPTH_TEST);

	renderer->setTextures(skyNegX);
	renderer->apply();
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex3f(-128,  128, -128);

		glTexCoord2f(1,0);
		glVertex3f(-128,  128,  128);

		glTexCoord2f(1,1);
		glVertex3f(-128, -128,  128);

		glTexCoord2f(0,1);
		glVertex3f(-128, -128, -128);
	glEnd();
	renderer->setTextures(skyPosX);
	renderer->apply();
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex3f( 128,  128,  128);

		glTexCoord2f(1,0);
		glVertex3f( 128,  128, -128);

		glTexCoord2f(1,1);
		glVertex3f( 128, -128, -128);

		glTexCoord2f(0,1);
		glVertex3f( 128, -128,  128);
	glEnd();
	/*renderer->setTextures(skyNegY);
	renderer->apply();
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex3f(-128, -128,  128);

		glTexCoord2f(1,0);
		glVertex3f( 128, -128,  128);

		glTexCoord2f(1,1);
		glVertex3f( 128, -128, -128);

		glTexCoord2f(0,1);
		glVertex3f(-128, -128, -128);
	glEnd();*/
	renderer->setTextures(skyPosY);
	renderer->apply();
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex3f(-128,  128, -128);

		glTexCoord2f(1,0);
		glVertex3f( 128,  128, -128);

		glTexCoord2f(1,1);
		glVertex3f( 128,  128,  128);

		glTexCoord2f(0,1);
		glVertex3f(-128,  128,  128);
	glEnd();
	renderer->setTextures(skyPosZ);
	renderer->apply();
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex3f(-128,  128,  128);

		glTexCoord2f(1,0);
		glVertex3f( 128,  128,  128);

		glTexCoord2f(1,1);
		glVertex3f( 128, -128,  128);

		glTexCoord2f(0,1);
		glVertex3f(-128, -128,  128);
	glEnd();
	renderer->setTextures(skyNegZ);
	renderer->apply();
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex3f( 128,  128, -128);

		glTexCoord2f(1,0);
		glVertex3f(-128,  128, -128);

		glTexCoord2f(1,1);
		glVertex3f(-128, -128, -128);

		glTexCoord2f(0,1);
		glVertex3f( 128, -128, -128);
	glEnd();

	modelView.translate(-position);
    glMatrixMode(GL_MODELVIEW);
	glLoadMatrixf(modelView);

	glEnable(GL_DEPTH_TEST);

    float fogColor[4] = { 58.0f / 255, 68.0f / 255, 184.0f / 255, 1.0f };

    glFogfv(GL_FOG_COLOR, fogColor);
    glFogi(GL_FOG_MODE, GL_EXP2);
    glFogf(GL_FOG_DENSITY, 0.001f);
    //glFogf(GL_FOG_START, 1);
    //glFogf(GL_FOG_END, 4000);
    glEnable(GL_FOG);

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	float lightDir[] = { -1.0f, 0.5f, 0.2f, 0.0f };
	glLightfv(GL_LIGHT0, GL_POSITION, lightDir);

	renderer->setTextures(ground);
	//renderer->setTexGen(TEXGEN_OBJECT_LINEAR);
	renderer->apply();

	/*renderer->changeTextureUnit(0);
	glMatrixMode(GL_TEXTURE);
	float mat[16] = {
		0.005f, 0,      0, 0,
		0,      0,      0, 0,
		0,      0.005f, 0, 0,
		0,      0,      0, 1
	};
	glLoadMatrixf(mat);
	*/


	int nVertices = terrain->draw(renderer, position, 4000, 0.0013f);

	//glLoadIdentity();

	char str[500];
	sprintf(str, "%.1f Mt/s %s", (nVertices / time) / 1000000, m_position_string);
	setDebugString(str);

	glDisable(GL_LIGHTING);
    glDisable(GL_FOG);

	return true;
}

DECLARE_MAIN_APP(MainApp)
